@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: hostlist.c,v 1.5 2007/01/10 18:52:54 achu Exp $
+ *  $Id: hostlist.c,v 1.3 2008/01/29 19:09:05 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -65,7 +65,7 @@
 #endif /* !WITH_LSD_FATAL_ERROR_FUNC */
 
 /*
- * lsd_nonmem_error
+ * lsd_nomem_error
  */
 #ifdef WITH_LSD_NOMEM_ERROR_FUNC
 #  undef lsd_nomem_error
@@ -207,7 +207,7 @@ static char * _next_tok(char *, char **);
 static int    _zero_padded(unsigned long, int);
 static int    _width_equiv(unsigned long, int *, unsigned long, int *);
 
-static size_t        host_prefix_end(const char *);
+static int           host_prefix_end(const char *);
 static hostname_t    hostname_create(const char *);
 static void          hostname_destroy(hostname_t);
 static int           hostname_suffix_is_valid(hostname_t);
@@ -470,9 +470,9 @@ static int _width_equiv(unsigned long n, int *wn, unsigned long m, int *wm)
 /* 
  * return the location of the last char in the hostname prefix
  */
-static size_t host_prefix_end(const char *hostname)
+static int host_prefix_end(const char *hostname)
 {
-    size_t idx = strlen(hostname) - 1;
+    int idx = strlen(hostname) - 1;
 
     while (idx >= 0 && isdigit((char) hostname[idx])) 
         idx--;
@@ -486,7 +486,7 @@ static hostname_t hostname_create(const char *hostname)
 {
     hostname_t hn = NULL;
     char *p = '\0';
-    size_t idx = 0;
+    int idx = 0;
 
     assert(hostname != NULL);
 
@@ -1421,6 +1421,10 @@ _push_range_list_with_suffix(hostlist_t hl, char *pfx, char *sfx,
             snprintf (host, 4096, "%s%0*lu%s", pfx, rng->width, j, sfx);
             hr = hostrange_create_single (host);
             hostlist_push_range (hl, hr);
+            /*
+             * hr is copied in hostlist_push_range. Need to free here.
+             */
+            hostrange_destroy (hr);
         }
         rng++;
     }
@@ -1987,6 +1991,7 @@ static void hostlist_coalesce(hostlist_t hl)
 /* attempt to join ranges at loc and loc-1 in a hostlist  */
 /* delete duplicates, return the number of hosts deleted  */
 /* assumes that the hostlist hl has been locked by caller */
+/* returns -1 if no range join occurred */
 static int _attempt_range_join(hostlist_t hl, int loc)
 {
     int ndup;
@@ -2026,7 +2031,7 @@ void hostlist_uniq(hostlist_t hl)
 }
 
 
-size_t hostlist_deranged_string(hostlist_t hl, size_t n, char *buf)
+ssize_t hostlist_deranged_string(hostlist_t hl, size_t n, char *buf)
 {
     int i;
     int len = 0;
@@ -2115,7 +2120,7 @@ _get_bracketed_list(hostlist_t hl, int *start, const size_t n, char *buf)
     return len;
 }
 
-size_t hostlist_ranged_string(hostlist_t hl, size_t n, char *buf)
+ssize_t hostlist_ranged_string(hostlist_t hl, size_t n, char *buf)
 {
     int i = 0;
     int len = 0;
@@ -2262,7 +2267,8 @@ char *hostlist_next(hostlist_iterator_t i)
         snprintf (suffix, 15, "%0*lu", i->hr->width, i->hr->lo + i->depth);
 
     len = strlen (i->hr->prefix) + strlen (suffix) + 1;
-    buf = malloc (len);
+    if (!(buf = malloc (len)))
+        out_of_memory("hostlist_next");
     
     buf[0] = '\0';
     strcat (buf, i->hr->prefix);
@@ -2366,12 +2372,14 @@ void hostset_destroy(hostset_t set)
 
 /* inserts a single range object into a hostset 
  * Assumes that the set->hl lock is already held
+ * Updates hl->nhosts 
  */
 static int hostset_insert_range(hostset_t set, hostrange_t hr)
 {
-    int i, n = 0;
+    int i = 0;
     int inserted = 0;
-    int retval = 0;
+    int nhosts = 0;
+    int ndups = 0;
     hostlist_t hl;
 
     hl = set->hl;
@@ -2379,24 +2387,25 @@ static int hostset_insert_range(hostset_t set, hostrange_t hr)
     if (hl->size == hl->nranges && !hostlist_expand(hl))
         return 0;
 
-    retval = hostrange_count(hr);
+    nhosts = hostrange_count(hr);
 
     for (i = 0; i < hl->nranges; i++) {
         if (hostrange_cmp(hr, hl->hr[i]) <= 0) {
-            n = hostrange_join(hr, hl->hr[i]);
 
-            if (n >= 0) {
+            if ((ndups = hostrange_join(hr, hl->hr[i])) >= 0) 
                 hostlist_delete_range(hl, i);
-                hl->nhosts -= n;
-            }
+            else if (ndups < 0)
+                ndups = 0;
 
             hostlist_insert_range(hl, hr, i);
 
             /* now attempt to join hr[i] and hr[i-1] */
             if (i > 0) {
-                int m = _attempt_range_join(hl, i);
-                n += m;
+                int m;
+                if ((m = _attempt_range_join(hl, i)) > 0)
+                    ndups += m;
             }
+            hl->nhosts += nhosts - ndups;
             inserted = 1;
             break;
         }
@@ -2404,10 +2413,17 @@ static int hostset_insert_range(hostset_t set, hostrange_t hr)
 
     if (inserted == 0) {
         hl->hr[hl->nranges++] = hostrange_copy(hr);
-        n = _attempt_range_join(hl, hl->nranges - 1);
-    }
+        hl->nhosts += nhosts;
+        if (hl->nranges > 1) {
+            if ((ndups = _attempt_range_join(hl, hl->nranges - 1)) <= 0)
+                ndups = 0;
+        }
+    } 
 
-    return retval - n;
+    /* 
+     *  Return the number of unique hosts inserted 
+     */
+    return nhosts - ndups;
 }
 
 int hostset_insert(hostset_t set, const char *hosts)
@@ -2456,7 +2472,9 @@ int hostset_within(hostset_t set, const char *hosts)
 
     assert(set->hl->magic == HOSTLIST_MAGIC);
 
-    hl = hostlist_create(hosts);
+    if (!(hl = hostlist_create(hosts)))
+        return (0);
+
     nhosts = hostlist_count(hl);
     nfound = 0;
 
@@ -2505,12 +2523,12 @@ int hostset_count(hostset_t set)
     return hostlist_count(set->hl);
 }
 
-size_t hostset_ranged_string(hostset_t set, size_t n, char *buf)
+ssize_t hostset_ranged_string(hostset_t set, size_t n, char *buf)
 {
     return hostlist_ranged_string(set->hl, n, buf);
 }
 
-size_t hostset_deranged_string(hostset_t set, size_t n, char *buf)
+ssize_t hostset_deranged_string(hostset_t set, size_t n, char *buf)
 {
     return hostlist_deranged_string(set->hl, n, buf);
 }
