@@ -1,7 +1,8 @@
 /*****************************************************************************\
- *  $Id: genders_test_functionality.c,v 1.8 2007/01/10 18:52:54 achu Exp $
+ *  $Id: genders_test_functionality.c,v 1.10 2008/03/28 16:56:40 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2001-2003 The Regents of the University of California.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2001-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov> and Albert Chu <chu11@llnl.gov>.
  *  UCRL-CODE-2003-004.
@@ -20,8 +21,7 @@
  *  details.
  *
  *  You should have received a copy of the GNU General Public License along
- *  with Genders; if not, write to the Free Software Foundation, Inc.,
- *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
+ *  with Genders.  If not, see <http://www.gnu.org/licenses/>.
 \*****************************************************************************/
 
 #include <stdio.h>
@@ -1483,8 +1483,8 @@ genders_testattr_functionality(int verbose)
       
       err = genders_return_value_errnum_check("genders_testattr",
 					      num,
-					      0,
-					      GENDERS_ERR_SUCCESS,
+                                              (databases[i]->data->node) ? 0 : -1,
+                                              (databases[i]->data->node) ? GENDERS_ERR_SUCCESS : GENDERS_ERR_NOTFOUND,
 					      return_value,
 					      errnum,
 					      databases[i]->filename,
